@@ -1,22 +1,28 @@
 #| -*-Scheme-*-
 
-$Id: lapgn2.scm,v 1.21 1999/01/02 06:06:43 cph Exp $
+$Id: lapgn2.scm,v 1.28 2008/01/30 20:01:42 cph Exp $
 
-Copyright (c) 1987-1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008 Massachusetts Institute of Technology
 
-This program is free software; you can redistribute it and/or modify
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
 |#
 
 ;;;; LAP Generator: High-Level Register Assignment
@@ -159,6 +165,16 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
       (and (register-type? register type) register)
       (maybe-need-register!
        (pseudo-register-alias *register-map* type register))))
+
+(define (guarantee-registers-compatible r1 r2)
+  (if (not (registers-compatible? r1 r2))
+      (error "Incompatible register types:" r1 r2)))
+
+(define (registers-compatible? r1 r2)
+  (register-types-compatible? (register-type r1) (register-type r2)))
+
+(define (register-types-compatible? type1 type2)
+  (boolean=? (eq? type1 'FLOAT) (eq? type2 'FLOAT)))
 
 (define (load-alias-register! register type)
   ;; Returns an alias for `register', of the given `type'.  If no such
