@@ -1,22 +1,29 @@
-;;; -*-Scheme-*-
-;;;
-;;; $Id: os2term.scm,v 1.22 2000/12/01 06:07:26 cph Exp $
-;;;
-;;; Copyright (c) 1994-2000 Massachusetts Institute of Technology
-;;;
-;;; This program is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU General Public License as
-;;; published by the Free Software Foundation; either version 2 of the
-;;; License, or (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#| -*-Scheme-*-
+
+$Id: os2term.scm,v 1.29 2007/01/05 21:19:24 cph Exp $
+
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007 Massachusetts Institute of Technology
+
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
+|#
 
 ;;;; OS/2 Presentation Manager Interface
 ;;; Package: (edwin screen os2-screen)
@@ -89,7 +96,7 @@
 		  (lambda ()
 		    (receiver (lambda (thunk) (thunk)) '()))
 		  (lambda ()
-		    (deregister-input-thread-event previewer-registration)))))
+		    (deregister-io-thread-event previewer-registration)))))
 
 (define (with-os2-interrupts-enabled thunk)
   (with-signal-interrupts #t thunk))
@@ -725,8 +732,9 @@
 		 event:process-status)
 		(else
 		 (let ((flag
-			(test-for-input-on-descriptor event-descriptor
-						      block?)))
+			(test-for-io-on-descriptor event-descriptor
+						   block?
+						   'READ)))
 		   (set-interrupt-enables! interrupt-mask)
 		   (case flag
 		     ((#F) #f)
@@ -740,10 +748,12 @@
 
 (define (preview-event-stream)
   (set! previewer-registration
-	(permanently-register-input-thread-event
+	(permanently-register-io-thread-event
 	 event-descriptor
+	 'READ
 	 (current-thread)
-	 (lambda ()
+	 (lambda (mode)
+	   mode
 	   (if (not reading-event?)
 	       (let ((event (os2win-get-event event-descriptor #f)))
 		 (if event
@@ -797,16 +807,17 @@
 (define (translate-key-event event)
   (let ((code (key-event/code event))
 	(flags (key-event/flags event)))
-    (let ((control (if (fix:= 0 (fix:and flags KC_CTRL)) 0 2))
-	  (meta (if (fix:= 0 (fix:and flags KC_ALT)) 0 1)))
+    (let ((bits (flags->bucky-bits flags)))
       (let ((process-code
 	     (lambda (code)
-	       (if (and (fix:<= #o100 code) (fix:< code #o140)
-			(not (fix:= 0 control)))
-		   (make-char (fix:and code #o037) meta)
-		   (make-char code (fix:or meta control))))))
+	       (if (and (fix:<= #x40 code) (fix:< code #x60)
+			(fix:= (fix:and bits char-bit:control)
+			       char-bit:control))
+		   (make-char (fix:and code #x1F)
+			      (fix:andc bits char-bit:control))
+		   (make-char code bits)))))
 	(if (fix:= 0 (fix:and flags KC_VIRTUALKEY))
-	    (and (fix:< code #o200)
+	    (and (fix:< code #x80)
 		 (process-code code))
 	    (let ((key
 		   (and (fix:< code (vector-length virtual-key-table))
@@ -814,7 +825,7 @@
 	      (and key
 		   (if (fix:fixnum? key)
 		       (process-code key)
-		       (make-special-key key (fix:or meta control))))))))))
+		       (make-special-key key bits)))))))))
 
 (define (process-change-event event)
   (cond ((fix:= event event:process-output) (accept-process-output))
@@ -887,12 +898,14 @@
   (lambda (screen event)
     (and (eq? button-event-type:down (button-event/type event))
 	 (if (os2win-focus? (screen-wid screen))
-	     (make-input-event 'BUTTON
-			       execute-button-command
-			       screen
-			       (make-down-button (button-event/number event))
-			       (x->cx screen (button-event/x event))
-			       (y->cy screen (button-event/y event)))
+	     (make-input-event
+	      'BUTTON
+	      execute-button-command
+	      screen
+	      (make-down-button (button-event/number event)
+				(flags->bucky-bits (button-event/flags event)))
+	      (x->cx screen (button-event/x event))
+	      (y->cy screen (button-event/y event)))
 	     (begin
 	       (os2win-activate (screen-wid screen))
 	       #f)))))
@@ -1018,4 +1031,8 @@
     (vector-set! table VK_CLEAR		'CLEAR)
     (vector-set! table VK_EREOF		'EREOF)
     (vector-set! table VK_PA1		'PA1)
-    table))
\ No newline at end of file
+    table))
+
+(define (flags->bucky-bits flags)
+  (fix:or (if (fix:= 0 (fix:and flags KC_CTRL)) #x2 #x0)
+	  (if (fix:= 0 (fix:and flags KC_ALT))  #x1 #x0)))
\ No newline at end of file
