@@ -1,23 +1,29 @@
-;;; -*-Scheme-*-
-;;;
-;;; $Id: sendmail.scm,v 1.78 2001/04/11 01:28:28 cph Exp $
-;;;
-;;; Copyright (c) 1991-2001 Massachusetts Institute of Technology
-;;;
-;;; This program is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU General Public License as
-;;; published by the Free Software Foundation; either version 2 of the
-;;; License, or (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;;; 02111-1307, USA.
+#| -*-Scheme-*-
+
+$Id: sendmail.scm,v 1.92 2007/09/09 16:36:50 riastradh Exp $
+
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007 Massachusetts Institute of Technology
+
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
+|#
 
 ;;;; Mail Sending
 
@@ -83,7 +89,7 @@ so you can remove or alter the BCC field to override the default."
   "Name of file to write all outgoing messages in, or #f for none."
   #f
   string-or-false?)
-
+
 (define-variable mail-relay-host
   "Name of host to which all outgoing mail should be sent.
 Can be a host name (a string) or #F.
@@ -92,6 +98,17 @@ Otherwise, mail is sent directly to the named host using SMTP."
   #f
   string-or-false?)
 
+(define-variable mail-relay-service
+  "Service to connect to on the mail relay host.
+Can be a service name (a string), a service number, or #F.
+If #F, service \"smtp\" is used.
+This is used only of `mail-relay-host' is set."
+  #f
+  (lambda (service)
+    (or (not service)
+        (exact-positive-integer? service)
+        (string? service))))
+
 (define-variable smtp-trace
   "If true, direct SMTP transmissions are traced in a buffer."
   #f
@@ -102,24 +119,36 @@ Otherwise, mail is sent directly to the named host using SMTP."
 Otherwise, only one valid recipient is required."
   #t
   boolean?)
+
+(define-variable sendmail-program
+  "Filename of sendmail program."
+  #f
+  string-or-false?)
+
+(define-variable send-mail-procedure
+  "Procedure to call to send the current buffer as mail.
+The headers are delimited by a string found in mail-header-separator."
+  (lambda () (sendmail-send-it))
+  (lambda (object)
+    (and (procedure? object)
+	 (procedure-arity-valid? object 0))))
+(variable-permanent-local! (ref-variable-object send-mail-procedure))
 
 (define-variable mail-yank-ignored-headers
   "Delete these headers from old message when it's inserted in a reply."
-  (reduce (lambda (x y) (string-append x "\\|" y))
-	  ""
-	  '("^via:"
-	    "^mail-from:"
-	    "^origin:"
-	    "^status:"
-	    "^remailed"
-	    "^received:"
-	    "^[a-z-]*message-id:"
-	    "^summary-line:"
-	    "^to:"
-	    "^cc:"
-	    "^subject:"
-	    "^in-reply-to:"
-	    "^return-path:"))
+  (regexp-group "^via:"
+		"^mail-from:"
+		"^origin:"
+		"^status:"
+		"^remailed"
+		"^received:"
+		"^[a-z-]*message-id:"
+		"^summary-line:"
+		"^to:"
+		"^cc:"
+		"^subject:"
+		"^in-reply-to:"
+		"^return-path:")
   string?)
 
 (define-variable mail-interactive
@@ -144,20 +173,6 @@ is ignored."
 	(and (procedure? object)
 	     (procedure-arity-valid? object 1)))))
 
-(define-variable sendmail-program
-  "Filename of sendmail program."
-  (os/sendmail-program)
-  string?)
-
-(define-variable send-mail-procedure
-  "Procedure to call to send the current buffer as mail.
-The headers are delimited by a string found in mail-header-separator."
-  (lambda () (sendmail-send-it))
-  (lambda (object)
-    (and (procedure? object)
-	 (procedure-arity-valid? object 0))))
-(variable-permanent-local! (ref-variable-object send-mail-procedure))
-
 (define-variable mail-reply-buffer
   ""
   #f
@@ -308,7 +323,7 @@ is inserted."
     (let ((given-header?
 	   (lambda (name null-true?)
 	     (let ((header
-		    (list-search-positive headers
+		    (find-matching-item headers
 		      (lambda (header)
 			(string-ci=? (car header) name)))))
 	       (and header
@@ -689,9 +704,12 @@ the user from the mailer."
 		 (lambda (name value)
 		   (if (and value (not (mail-field-start h-start h-end name)))
 		       (mail-insert-field-value h-end name value)))))
+	    (add-field "Date" (universal-time->string (get-universal-time)))
 	    ;; If there is a From and no Sender, put in a Sender.
-	    (if (mail-field-start h-start h-end "From")
-		(add-field "Sender" (mail-from-string mail-buffer)))
+	    (add-field (if (mail-field-start h-start h-end "From")
+			   "Sender"
+			   "From")
+		       (mail-from-string mail-buffer))
 	    (add-field "Organization" (mail-organization-string mail-buffer))
 	    (add-field "User-Agent" (mailer-version-string mail-buffer)))
 	  (let ((v (receiver h-start h-end b-start b-end)))
@@ -738,14 +756,18 @@ the user from the mailer."
 	(if (ref-variable mail-relay-host lookup-context)
 	    (let ((recipients (compute-message-recipients h-start h-end)))
 	      (write-message-file h-start h-end b-start b-end message-pathname)
-	      (write-fcc-messages fcc-pathnames message-pathname)
+	      (write-fcc-messages fcc-pathnames
+				  message-pathname
+				  lookup-context)
 	      (lambda ()
 		(send-mail-using-smtp message-pathname
 				      recipients
 				      lookup-context)))
 	    (begin
 	      (write-message-file h-start h-end b-start b-end message-pathname)
-	      (write-fcc-messages fcc-pathnames message-pathname)
+	      (write-fcc-messages fcc-pathnames
+				  message-pathname
+				  lookup-context)
 	      (lambda ()
 		(send-mail-using-sendmail message-pathname lookup-context))))
 	message-pathname)))))
@@ -782,17 +804,18 @@ the user from the mailer."
 	    (mark-temporary! m)
 	    pathnames)))))
 
-(define (write-fcc-messages pathnames message-pathname)
+(define (write-fcc-messages pathnames message-pathname lookup-context)
   (for-each
    (let ((append-message
 	  (let ((header-line
 		 (string-append
 		  "From "
-		  (current-user-name)
+		  (user-mail-address lookup-context)
 		  " "
-		  (universal-time->string (get-universal-time)))))
-	    (lambda (port)
-	      (newline port)
+		  (universal-time->local-ctime-string (get-universal-time)))))
+	    (lambda (length port)
+	      (if (> length 0)
+		  (newline port))
 	      (write-string header-line port)
 	      (newline port)
 	      (call-with-input-file message-pathname
@@ -815,39 +838,44 @@ the user from the mailer."
      (lambda (pathname)
        (let ((buffer (pathname->buffer pathname)))
 	 (if buffer
-	     (call-with-output-mark (buffer-end buffer) append-message)
-	     (call-with-append-file pathname append-message)))))
+	     (call-with-output-mark (buffer-end buffer)
+	       (lambda (port)
+		 (append-message (buffer-length buffer) port)))
+	     (call-with-append-file pathname
+	       (lambda (port)
+		 (append-message ((port/operation port 'LENGTH) port)
+				 port)))))))
    pathnames))
 
 (define (compute-message-recipients h-start h-end)
-  (call-with-values
-      (lambda ()
-	(if (mail-field-start h-start h-end "resent-to")
-	    (values "^\\(resent-to:\\|resent-cc:\\|resent-bcc:\\)[ \t]*"
-		    "resent-bcc:")
-	    (values "^\\(to:\\|cc:\\|bcc:\\)[ \t]*" "bcc:")))
-    (lambda (regexp prefix)
-      (let loop ((start h-start) (addresses '()))
-	(let ((f-start (re-search-forward regexp start h-end #t)))
-	  (if f-start
-	      (let* ((f-end (%mail-field-end f-start h-end))
-		     (addresses
-		      (append (rfc822:string->addresses
-			       (extract-string f-start f-end))
-			      addresses))
-		     (ls (line-start f-start 0)))
-		(if (match-forward prefix ls h-end #t)
-		    (begin
-		      (delete-string ls (mark1+ f-end 'LIMIT))
-		      (loop ls addresses))
-		    (loop f-end addresses)))
-	      addresses))))))
+  (receive (regexp prefix)
+      (if (mail-field-start h-start h-end "resent-to")
+	  (values "^\\(resent-to:\\|resent-cc:\\|resent-bcc:\\)[ \t]*"
+		  "resent-bcc:")
+	  (values "^\\(to:\\|cc:\\|bcc:\\)[ \t]*" "bcc:"))
+    (let loop ((start h-start) (addresses '()))
+      (let ((f-start (re-search-forward regexp start h-end #t)))
+	(if f-start
+	    (let* ((f-end (%mail-field-end f-start h-end))
+		   (addresses
+		    (append (rfc822:string->addresses
+			     (extract-string f-start f-end))
+			    addresses))
+		   (ls (line-start f-start 0)))
+	      (if (match-forward prefix ls h-end #t)
+		  (begin
+		    (delete-string ls (mark1+ f-end 'LIMIT))
+		    (loop ls addresses))
+		  (loop f-end addresses)))
+	    addresses)))))
 
 ;;;; Sendmail transmission
 
 (define (send-mail-using-sendmail message-pathname lookup-context)
   (message "Sending...")
-  (let ((program (ref-variable sendmail-program lookup-context)))
+  (let ((program
+	 (or (ref-variable sendmail-program lookup-context)
+	     (os/sendmail-program))))
     (if (ref-variable mail-interactive lookup-context)
 	(call-with-temporary-buffer " sendmail errors"
 	  (lambda (error-buffer)
@@ -904,6 +932,8 @@ the user from the mailer."
 	(editor-error "No recipients specified for mail."))
     (let ((responses
 	   (call-with-smtp-socket (ref-variable mail-relay-host lookup-context)
+                                  (ref-variable mail-relay-service
+                                                lookup-context)
 				  trace-buffer
 	     (lambda (port banner)
 	       banner
@@ -946,12 +976,13 @@ the user from the mailer."
 		   "done"
 		   "aborted")))))
 
-(define (call-with-smtp-socket host-name trace-buffer receiver)
+(define (call-with-smtp-socket host-name service trace-buffer receiver)
   (let ((port #f))
     (dynamic-wind
      (lambda ()
        (set! port
-	     (make-smtp-port (open-tcp-stream-socket host-name "smtp")
+	     (make-smtp-port (open-tcp-stream-socket host-name
+                                                     (or service "smtp"))
 			     trace-buffer))
        unspecific)
      (lambda ()
@@ -1101,7 +1132,11 @@ the user from the mailer."
 			(mark-index b-start)
 			(mark-index b-end)
 	    (lambda (string start end)
-	      (encode-quoted-printable:update context string start end)))
+	      (encode-quoted-printable:update
+               context
+               (xsubstring string 0 (xstring-length string))
+               start
+               end)))
 	  (encode-quoted-printable:finalize context)))
       (begin
 	(if (not (and subpart? (ref-variable mail-abbreviate-mime b-start)))
@@ -1195,32 +1230,30 @@ the user from the mailer."
 		    (mime-attachment-message-headers attachment))
 	  (newline port)
 	  ((mime-attachment-message-body-generator attachment) port))
-	(call-with-values
-	    (lambda ()
-	      (if (eq? type 'TEXT)
-		  (values encode-quoted-printable:initialize
-			  encode-quoted-printable:update
-			  encode-quoted-printable:finalize
-			  #t)
-		  (values encode-base64:initialize
-			  encode-base64:update
-			  encode-base64:finalize
-			  #f)))
-	  (lambda (initialize update finalize text?)
-	    (let ((context (initialize port text?)))
-	      ((if (eq? type 'TEXT)
-		   call-with-input-file
-		   call-with-binary-input-file)
-	       (mime-attachment-pathname attachment)
-	       (lambda (input-port)
-		 (let ((buffer (make-string 4096)))
-		   (let loop ()
-		     (let ((n-read (read-string! buffer input-port)))
-		       (if (> n-read 0)
-			   (begin
-			     (update context buffer 0 n-read)
-			     (loop))))))))
-	      (finalize context)))))))
+	(receive (initialize update finalize text?)
+	    (if (eq? type 'TEXT)
+		(values encode-quoted-printable:initialize
+			encode-quoted-printable:update
+			encode-quoted-printable:finalize
+			#t)
+		(values encode-base64:initialize
+			encode-base64:update
+			encode-base64:finalize
+			#f))
+	  (let ((context (initialize port text?)))
+	    ((if (eq? type 'TEXT)
+		 call-with-input-file
+		 call-with-binary-input-file)
+	     (mime-attachment-pathname attachment)
+	     (lambda (input-port)
+	       (let ((buffer (make-string 4096)))
+		 (let loop ()
+		   (let ((n-read (read-string! buffer input-port)))
+		     (if (> n-read 0)
+			 (begin
+			   (update context buffer 0 n-read)
+			   (loop))))))))
+	    (finalize context))))))
 
 (define (enable-buffer-mime-processing! buffer)
   (buffer-remove! buffer 'MAIL-DISABLE-MIME-PROCESSING))
@@ -1231,11 +1264,15 @@ the user from the mailer."
 (define (buffer-mime-processing-enabled? buffer)
   (not (buffer-get buffer 'MAIL-DISABLE-MIME-PROCESSING #f)))
 
-(define (add-buffer-mime-attachment! buffer
-				     type subtype parameters disposition
+(define (add-buffer-mime-attachment! buffer mime-type parameters disposition
 				     . rest)
   (let ((attachment
-	 (list->vector (cons* type subtype parameters disposition rest))))
+	 (list->vector
+	  (cons* (mime-type/top-level mime-type)
+		 (mime-type/subtype mime-type)
+		 parameters
+		 disposition
+		 rest))))
     (set-buffer-mime-attachments! buffer
 				  (append (buffer-mime-attachments buffer)
 					  (list attachment)))
@@ -1333,13 +1370,13 @@ the user from the mailer."
     (if (not (<= 1 length (- 70 plen)))
 	(error:bad-range-argument length 'RANDOM-MIME-BOUNDARY-STRING))
     (let ((s
-	   (with-string-output-port
-	     (lambda (port)
-	       (write-string prefix port)
-	       (let ((context (encode-base64:initialize port #f)))
-		 (let ((n (* (integer-ceiling (- length 2) 4) 3)))
-		   (encode-base64:update context (random-byte-vector n) 0 n))
-		 (encode-base64:finalize context)))))
+	   (call-with-output-string
+	    (lambda (port)
+	      (write-string prefix port)
+	      (let ((context (encode-base64:initialize port #f)))
+		(let ((n (* (integer-ceiling (- length 2) 4) 3)))
+		  (encode-base64:update context (random-byte-vector n) 0 n))
+		(encode-base64:finalize context)))))
 	  (n (+ plen length)))
       (if (fix:> (string-length s) n)
 	  (set-string-maximum-length! s n))
@@ -1476,16 +1513,15 @@ Otherwise, the MIME type is determined from the file's suffix;
   (lambda (pathname argument)
     (let ((mail-buffer (selected-mail-buffer)))
       (let ((attachment
-	     (call-with-values
-		 (lambda ()
-		   (pathname->mime-type pathname mail-buffer argument))
-	       (lambda (type subtype parameters)
-		 (add-buffer-mime-attachment!
-		  mail-buffer type subtype
-		  `(,@parameters
-		    (NAME ,(pathname-name pathname)))
-		  `(ATTACHMENT (FILENAME ,(file-namestring pathname)))
-		  pathname)))))
+	     (receive (mime-type parameters)
+		 (pathname->mime-type pathname mail-buffer argument)
+	       (add-buffer-mime-attachment!
+		mail-buffer
+		mime-type
+		`(,@parameters
+		  (NAME ,(pathname-name pathname)))
+		`(ATTACHMENT (FILENAME ,(file-namestring pathname)))
+		pathname))))
 	(let ((buffer (get-mime-attachments-buffer mail-buffer #f)))
 	  (if buffer
 	      (let ((mark (mark-left-inserting-copy (buffer-end buffer))))
@@ -1523,66 +1559,46 @@ Otherwise, the MIME type is determined from the file's suffix;
       (kill-buffer-interactive buffer))))
 
 (define (pathname->mime-type pathname buffer prompt?)
-  (let ((type (pathname-type pathname))
-	(finish
-	 (lambda (type subtype)
-	   (values type
-		   subtype
-		   (if (eq? type 'TEXT)
-		       '((CHARSET "iso-8859-1"))
-		       '())))))
-    (let ((do-mime
-	   (lambda ()
-	     (let ((type
-		    (prompt-for-alist-value "MIME type"
-					    mime-top-level-types
-					    #f
-					    #t)))
-	       (finish type
+  (let ((mime-type
+	 (let ((type (pathname-type pathname)))
+	   (let ((do-mime
+		  (lambda ()
+		    (let ((type
+			   (prompt-for-alist-value "MIME type"
+						   mime-top-level-types
+						   #f
+						   #t)))
+		      (make-mime-type
+		       type
 		       (string->symbol
 			(prompt-for-string "MIME subtype" #f)))))))
-      (if prompt?
-	  (do-mime)
-	  (let ((entry
-		 (list-search-positive
-		     (ref-variable file-type-to-mime-type buffer)
-		   (lambda (entry)
-		     (if type
-			 (string-ci=? (car entry) type)
-			 (not (car entry)))))))
-	    (cond (entry
-		   (finish (cadr entry) (caddr entry)))
-		  ((search-mime-types-file pathname)
-		   => (lambda (ts) (finish (car ts) (cadr ts))))
-		  (else
-		   (let loop ()
-		     (case (prompt-for-char
-			    "File type (T=text, B=binary, M=MIME)")
-		       ((#\t #\T) (finish 'TEXT 'PLAIN))
-		       ((#\b #\B) (finish 'APPLICATION 'OCTET-STREAM))
-		       ((#\m #\M) (do-mime))
-		       (else (editor-beep) (loop)))))))))))
-
-(define (search-mime-types-file pathname)
-  (let ((filename (file-namestring pathname)))
-    (call-with-input-file (system-library-pathname "edwin/etc/mime.types")
-      (lambda (port)
-	(let loop ()
-	  (let ((line (read-line port)))
-	    (and (not (eof-object? line))
-		 (let ((line (string-trim line)))
-		   (if (or (string-null? line)
-			   (char=? (string-ref line 0) #\#))
-		       (loop)
-		       (let ((tokens
-			      (burst-string line char-set:whitespace #t)))
-			 (if (there-exists? (cdr tokens)
-			       (lambda (suffix)
-				 (string-suffix-ci? (string-append "." suffix)
-						    filename)))
-			     (map intern
-				  (burst-string (car tokens) #\/ #f))
-			     (loop))))))))))))
+	     (if prompt?
+		 (do-mime)
+		 (let ((entry
+			(find-matching-item
+			    (ref-variable file-type-to-mime-type buffer)
+			  (lambda (entry)
+			    (cond ((string? type)
+				   (string-ci=? (car entry) type))
+				  ((not type)
+				   (not (car entry)))
+				  (else
+				   (eq? type 'WILD)))))))
+		   (cond (entry (make-mime-type (cadr entry) (caddr entry)))
+			 ((pathname-mime-type pathname))
+			 (else
+			  (let loop ()
+			    (case (prompt-for-char
+				   "File type (T=text, B=binary, M=MIME)")
+			      ((#\t #\T) (make-mime-type 'TEXT 'PLAIN))
+			      ((#\b #\B) (make-mime-type 'APPLICATION
+							 'OCTET-STREAM))
+			      ((#\m #\M) (do-mime))
+			      (else (editor-beep) (loop))))))))))))
+    (values mime-type
+	    (if (eq? (mime-type/top-level mime-type) 'TEXT)
+		'((CHARSET "iso-8859-1"))
+		'()))))
 
 (define-variable file-type-to-mime-type
   "Specifies the MIME type/subtype for files with a given type.
