@@ -1,23 +1,29 @@
-;;; -*-Scheme-*-
-;;;
-;;; $Id: imail-file.scm,v 1.80 2001/09/28 00:41:21 cph Exp $
-;;;
-;;; Copyright (c) 1999-2001 Massachusetts Institute of Technology
-;;;
-;;; This program is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU General Public License as
-;;; published by the Free Software Foundation; either version 2 of the
-;;; License, or (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;;; 02111-1307, USA.
+#| -*-Scheme-*-
+
+$Id: imail-file.scm,v 1.93 2008/01/30 20:02:09 cph Exp $
+
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008 Massachusetts Institute of Technology
+
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
+|#
 
 ;;;; IMAIL mail reader: file-based folder support
 
@@ -46,12 +52,15 @@
 	(hash-table/datum-list file-folder-types))))
 
 (define (url-file-folder-type url)
+  (or (file-folder-type (pathname-url-pathname url))
+      (error "Unknown folder type:" url)))
+
+(define (file-folder-type pathname)
   (let loop ((types (hash-table/datum-list file-folder-types)))
-    (if (not (pair? types))
-	(error "Unknown folder type:" url))
-    (if ((file-folder-type-predicate (car types)) url)
-	(car types)
-	(loop (cdr types)))))
+    (and (pair? types)
+	 (if ((file-folder-type-predicate (car types)) pathname)
+	     (car types)
+	     (loop (cdr types))))))
 
 ;;;; URL
 
@@ -146,7 +155,7 @@
 
 (define-method folder-url-is-selectable? ((url <file-url>))
   (and (url-exists? url)
-       (url-file-folder-type url)
+       (file-folder-type (pathname-url-pathname url))
        #t))
 
 (define-method url-corresponding-container ((url <file-url>))
@@ -340,8 +349,9 @@
 		       (let ((m (vector-ref messages i)))
 			 (if (message-deleted? m)
 			     (begin
-			       (detach-message! m)
-			       (object-modified! folder 'EXPUNGE i*)
+			       (let ((key (message-order-key m)))
+                                 (detach-message! m)
+                                 (object-modified! folder 'EXPUNGE i* key))
 			       (loop (fix:+ i 1) i*))
 			     (begin
 			       (set-message-index! m i*)
@@ -349,7 +359,7 @@
 			       (loop (fix:+ i 1) (fix:+ i* 1)))))))
 		 (set-file-folder-messages! folder messages*)))))))))
 
-(define-method search-folder ((folder <file-folder>) criteria)
+(define-method %search-folder ((folder <file-folder>) criteria)
   (cond ((string? criteria)
 	 (let ((n (folder-length folder)))
 	   (let loop ((index 0) (winners '()))
@@ -390,7 +400,7 @@
 
 (define-method save-resource ((folder <file-folder>))
   (and (let ((status (folder-sync-status folder)))
-	 (or (memq status '(FOLDER-MODIFIED PERSISTENT-DELETED))
+	 (or (memq status '(CACHE-MODIFIED PERSISTENT-DELETED))
 	     (and (eq? status 'BOTH-MODIFIED)
 		  (imail-ui:prompt-for-yes-or-no?
 		   "Disk file has changed since last read.  Save anyway"))))
@@ -454,10 +464,6 @@
   folder
   unspecific)
 
-(define-method folder-supports-mime? ((folder <file-folder>))
-  folder
-  #f)
-
 (define-method preload-folder-outlines ((folder <file-folder>))
   folder
   unspecific)
@@ -507,7 +513,7 @@
 	     (xsubstring (file-message-xstring message)
 			 (file-external-ref/start item)
 			 (file-external-ref/end item)))
-	    (call-next-method message))))))
+            item)))))
 
 (define-file-external-message-method message-header-fields
   <file-message>
@@ -522,9 +528,9 @@
   (lambda (s) s))
 
 (define-method file-message-body ((message <message>))
-  (with-string-output-port
-    (lambda (port)
-      (write-message-body message port))))
+  (call-with-output-string
+   (lambda (port)
+     (write-message-body message port))))
 
 (define-method write-message-body ((message <file-message>) port)
   (write-string (file-message-body message) port))
