@@ -1,23 +1,29 @@
-;;; -*-Scheme-*-
-;;;
-;;; $Id: imail-util.scm,v 1.39 2001/06/09 00:29:48 cph Exp $
-;;;
-;;; Copyright (c) 1999-2001 Massachusetts Institute of Technology
-;;;
-;;; This program is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU General Public License as
-;;; published by the Free Software Foundation; either version 2 of the
-;;; License, or (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;;; 02111-1307, USA.
+#| -*-Scheme-*-
+
+$Id: imail-util.scm,v 1.50 2007/07/07 17:22:19 cph Exp $
+
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007 Massachusetts Institute of Technology
+
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
+|#
 
 ;;;; IMAIL mail reader: utilities
 
@@ -46,7 +52,7 @@
 	(let ((item (car items)))
 	  (set! items (cdr items))
 	  item)
-	(make-eof-object #f))))
+	(eof-object))))
 
 (define (cut-list! items predicate)
   (if (or (not (pair? items)) (predicate (car items)))
@@ -134,6 +140,17 @@
 	(loop (fix:- end 1))
 	end)))
 
+(define (skip-lwsp-until-newline string start end)
+  (let loop ((index start))
+    (cond ((= index end)
+           #f)
+          ((char-lwsp? (string-ref string index))
+           (loop (+ index 1)))
+          ((char=? (string-ref string index) #\newline)
+           (+ index 1))
+          (else
+           #f))))
+
 (define (quote-lines lines)
   (map (lambda (line)
 	 (string-append "\t" line))
@@ -152,9 +169,9 @@
 		    (if (default-object? line-ending) "\n" line-ending)))
 
 (define (substring->lines string start end #!optional line-ending)
-  (let ((line-ending (if (default-object? line-ending) "\n" line-ending))
-	(n (string-length line-ending)))
-    (let ((indexes (substring-search-all line-ending string start end)))
+  (let ((line-ending (if (default-object? line-ending) "\n" line-ending)))
+    (let ((indexes (substring-search-all line-ending string start end))
+          (n (string-length line-ending)))
       (if (pair? indexes)
 	  (begin
 	    (let loop ((start start) (indexes indexes))
@@ -427,138 +444,116 @@
 (define (open-xstring-input-port xstring position)
   (if (not (<= 0 position (external-string-length xstring)))
       (error:bad-range-argument position 'OPEN-XSTRING-INPUT-PORT))
-  (let ((state (make-xstring-input-state xstring position)))
+  (let ((state (make-istate xstring position position position)))
     (read-xstring-buffer state)
     (make-port xstring-input-type state)))
 
-(define-structure (xstring-input-state
-		   (constructor make-xstring-input-state (xstring position))
-		   (conc-name xstring-input-state/))
+(define-structure (istate
+		   (constructor make-istate
+				(xstring position buffer-start buffer-end))
+		   (conc-name istate-))
   xstring
   position
-  (buffer (make-string 65536) read-only #t)
-  (buffer-start position)
-  (buffer-end position))
+  (buffer (make-string #x10000) read-only #t)
+  buffer-start
+  buffer-end)
 
 (define (xstring-port/xstring port)
-  (xstring-input-state/xstring (port/state port)))
+  (istate-xstring (port/state port)))
 
 (define (xstring-port/position port)
-  (xstring-input-state/position (port/state port)))
+  (istate-position (port/state port)))
 
 (define (read-xstring-buffer state)
-  (let ((xstring (xstring-input-state/xstring state))
-	(start (xstring-input-state/position state)))
+  (let ((xstring (istate-xstring state))
+	(start (istate-position state)))
     (let ((xend (external-string-length xstring)))
       (and (< start xend)
-	   (let* ((buffer (xstring-input-state/buffer state))
+	   (let* ((buffer (istate-buffer state))
 		  (end (min (+ start (string-length buffer)) xend)))
 	     (without-interrupts
 	      (lambda ()
-		(set-xstring-input-state/buffer-start! state start)
-		(set-xstring-input-state/buffer-end! state end)
+		(set-istate-buffer-start! state start)
+		(set-istate-buffer-end! state end)
 		(xsubstring-move! xstring start end buffer 0)))
 	     #t)))))
-
-(define (xsubstring xstring start end)
-  (let ((buffer (make-string (- end start))))
-    (xsubstring-move! xstring start end buffer 0)
-    buffer))
 
 (define (xstring-input-port/discard-chars port delimiters)
   (let ((state (port/state port)))
-    (if (or (< (xstring-input-state/position state)
-	       (xstring-input-state/buffer-end state))
+    (if (or (< (istate-position state) (istate-buffer-end state))
 	    (read-xstring-buffer state))
 	(let loop ()
-	  (let* ((start (xstring-input-state/buffer-start state))
+	  (let* ((start (istate-buffer-start state))
 		 (index
 		  (substring-find-next-char-in-set
-		   (xstring-input-state/buffer state)
-		   (- (xstring-input-state/position state) start)
-		   (- (xstring-input-state/buffer-end state) start)
+		   (istate-buffer state)
+		   (- (istate-position state) start)
+		   (- (istate-buffer-end state) start)
 		   delimiters)))
 	    (if index
-		(set-xstring-input-state/position! state (+ start index))
+		(set-istate-position! state (+ start index))
 		(begin
-		  (set-xstring-input-state/position!
-		   state
-		   (xstring-input-state/buffer-end state))
+		  (set-istate-position! state (istate-buffer-end state))
 		  (if (read-xstring-buffer state)
 		      (loop)))))))))
 
 (define (xstring-input-port/read-string port delimiters)
   (let ((state (port/state port)))
-    (if (or (< (xstring-input-state/position state)
-	       (xstring-input-state/buffer-end state))
+    (if (or (< (istate-position state) (istate-buffer-end state))
 	    (read-xstring-buffer state))
 	(let loop ((prefix #f))
-	  (let* ((start (xstring-input-state/buffer-start state))
-		 (b (xstring-input-state/buffer state))
-		 (si (- (xstring-input-state/position state) start))
-		 (ei (- (xstring-input-state/buffer-end state) start))
+	  (let* ((start (istate-buffer-start state))
+		 (b (istate-buffer state))
+		 (si (- (istate-position state) start))
+		 (ei (- (istate-buffer-end state) start))
 		 (index (substring-find-next-char-in-set b si ei delimiters)))
 	    (if index
 		(begin
-		  (set-xstring-input-state/position! state (+ start index))
+		  (set-istate-position! state (+ start index))
 		  (let ((s (make-string (fix:- index si))))
 		    (substring-move! b si index s 0)
-		    (if prefix (string-append prefix s) s)))
+		    (if prefix
+			(string-append prefix s)
+			s)))
 		(begin
-		  (set-xstring-input-state/position!
-		   state
-		   (xstring-input-state/buffer-end state))
+		  (set-istate-position! state (istate-buffer-end state))
 		  (let ((s (make-string (fix:- ei si))))
 		    (substring-move! b si ei s 0)
-		    (let ((p (if prefix (string-append prefix s) s)))
+		    (let ((p
+			   (if prefix
+			       (string-append prefix s)
+			       s)))
 		      (if (read-xstring-buffer state)
 			  (loop p)
 			  p)))))))
-	(make-eof-object port))))
-
+	(eof-object))))
+
 (define xstring-input-type
   (make-port-type
-   (let ((read
-	  (lambda (port discard?)
-	    (let ((state (port/state port)))
-	      (let ((position (xstring-input-state/position state)))
-		(if (or (< position (xstring-input-state/buffer-end state))
-			(read-xstring-buffer state))
-		    (let ((char
-			   (string-ref
-			    (xstring-input-state/buffer state)
-			    (- position
-			       (xstring-input-state/buffer-start state)))))
-		      (if discard?
-			  (set-xstring-input-state/position!
-			   state (+ position 1)))
-		      char)
-		    (make-eof-object port))))))
-	 (xlength
-	  (lambda (state)
-	    (external-string-length (xstring-input-state/xstring state)))))
-     `((READ-CHAR ,(lambda (port) (read port #t)))
-       (PEEK-CHAR ,(lambda (port) (read port #f)))
-       (DISCARD-CHAR
-	,(lambda (port)
-	   (let* ((state (port/state port))
-		  (position (xstring-input-state/position state)))
-	     (if (< position (xlength state))
-		 (set-xstring-input-state/position! state (+ position 1))))))
-       (DISCARD-CHARS ,xstring-input-port/discard-chars)
-       (READ-STRING ,xstring-input-port/read-string)
-       (LENGTH ,(lambda (port) (xlength (port/state port))))
-       (EOF?
-	,(lambda (port)
-	   (let ((state (port/state port)))
-	     (>= (xstring-input-state/position state) (xlength state)))))
-       (CLOSE
-	,(lambda (port)
-	   (let ((state (port/state port)))
-	     (without-interrupts
-	      (lambda ()
-		(set-xstring-input-state/xstring! state #f)
-		(set-xstring-input-state/position! state 0)
-		(set-xstring-input-state/buffer-start! state 0)
-		(set-xstring-input-state/buffer-end! state 0))))))))
+   `((READ-CHAR
+      ,(lambda (port)
+	 (let ((state (port/state port)))
+	   (let ((position (istate-position state)))
+	     (if (or (< position (istate-buffer-end state))
+		     (read-xstring-buffer state))
+		 (let ((char
+			(string-ref (istate-buffer state)
+				    (- position (istate-buffer-start state)))))
+		   (set-istate-position! state (+ position 1))
+		   char)
+		 (eof-object))))))
+     (EOF?
+      ,(lambda (port)
+	 (let ((state (port/state port)))
+	   (>= (istate-position state)
+	       (external-string-length (istate-xstring state))))))
+     (CLOSE
+      ,(lambda (port)
+	 (let ((state (port/state port)))
+	   (without-interrupts
+	    (lambda ()
+	      (set-istate-xstring! state #f)
+	      (set-istate-position! state 0)
+	      (set-istate-buffer-start! state 0)
+	      (set-istate-buffer-end! state 0)))))))
    #f))
\ No newline at end of file
