@@ -1,23 +1,28 @@
 #| -*-Scheme-*-
 
-$Id: ntprm.scm,v 1.37 2001/12/18 18:39:42 cph Exp $
+$Id: ntprm.scm,v 1.49 2007/01/05 21:19:28 cph Exp $
 
-Copyright (c) 1992-2001 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007 Massachusetts Institute of Technology
 
-This program is free software; you can redistribute it and/or modify
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
+
 |#
 
 ;;;; Miscellaneous Win32 Primitives
@@ -106,6 +111,19 @@ USA.
 
 (define (file-time->universal-time time) (+ time epoch))
 (define (universal-time->file-time time) (- time epoch))
+
+(define (os/suffix-mime-type suffix)
+  (let* ((name (string-append "HKEY_CLASSES_ROOT\\." suffix))
+	 (key (win32-registry/open-key name #f)))
+    (and key
+	 (receive (type value)
+	     (win32-registry/get-value key "Content Type")
+	   (and type
+		(begin
+		  (if (not (eq? type 'REG_SZ))
+		      (error "Wrong value type in registry entry:"
+			     name))
+		  value))))))
 
 (define get-environment-variable)
 (define set-environment-variable!)
@@ -312,16 +330,16 @@ USA.
 	  (error "Unable to find Windows system root."))
       (pathname-new-directory (pathname-as-directory sysroot) '(ABSOLUTE)))))
 
-(define (os/file-end-of-line-translation pathname)
+(define (file-line-ending pathname)
   (if (let ((type (dos/fs-drive-type pathname)))
 	(or (string=? "NFS" (car type))
 	    (string=? "NtNfs" (car type))
 	    (string=? "Samba" (car type))))
-      #f
-      "\r\n"))
+      'LF
+      'CRLF))
 
-(define (os/default-end-of-line-translation)
-  "\r\n")
+(define (default-line-ending)
+  'CRLF)
 
 (define (dos/fs-drive-type pathname)
   ;; (system-name . [nfs-]mount-point)
@@ -447,55 +465,9 @@ USA.
 		       (set! port #f)
 		       unspecific))))))))))
 
-(define-structure (nt-select-registry (conc-name nt-select-registry/))
-  descriptors)
-
-(define (make-select-registry . descriptors)
-  (make-nt-select-registry descriptors))
-
-(define (add-to-select-registry! registry descriptor)
-  (if (not (memv descriptor (nt-select-registry/descriptors registry)))
-      (set-nt-select-registry/descriptors!
-       registry
-       (cons descriptor (nt-select-registry/descriptors registry)))))
-
-(define (remove-from-select-registry! registry descriptor)
-  (set-nt-select-registry/descriptors!
-   registry
-   (delv! descriptor (nt-select-registry/descriptors registry))))
-
-(define (select-registry-test registry block?)
-  (let ((descriptors (list->vector (nt-select-registry/descriptors registry))))
-    (let ((result
-	   ((ucode-primitive nt:waitformultipleobjects 3)
-	    descriptors #f block?)))
-      (cond ((and (fix:<= 0 result) (fix:< result (vector-length descriptors)))
-	     (list (vector-ref descriptors result)))
-	    ((fix:= result -1) #f)
-	    ((fix:= result -2) 'INTERRUPT)
-	    ((fix:= result -3) 'PROCESS-STATUS-CHANGE)
-	    (else (error "Illegal result from select-internal:" result))))))
-
-(define (select-descriptor descriptor block?)
-  (let ((result
-	 ((ucode-primitive nt:waitformultipleobjects 3)
-	  (vector descriptor) #f block?)))
-    (case result
-      ((0) 'INPUT-AVAILABLE)
-      ((-1) #f)
-      ((-2) 'INTERRUPT)
-      ((-3) 'PROCESS-STATUS-CHANGE)
-      (else (error "Illegal result from select-internal:" result)))))
-
-(define console-channel-descriptor)
-
-(define (cache-console-channel-descriptor!)
-  (set! console-channel-descriptor
-	(channel-descriptor-for-select (tty-input-channel)))
-  unspecific)
-
 ;;;; Subprocess/Shell Support
 
+(define console-channel-descriptor)
 (define nt/hide-subprocess-windows?)
 (define nt/subprocess-argument-quote-char)
 (define nt/subprocess-argument-escape-char)
@@ -504,12 +476,14 @@ USA.
   (let ((reset!
 	 (lambda ()
 	   (reset-environment-variables!)
-	   (cache-console-channel-descriptor!))))
+	   (set! console-channel-descriptor
+		 (channel-descriptor-for-select (tty-input-channel)))
+	   unspecific)))
     (reset!)
     (add-event-receiver! event:after-restart reset!))
   (set! nt/hide-subprocess-windows? #t)
-  (set! nt/subprocess-argument-quote-char #f)
-  (set! nt/subprocess-argument-escape-char #f)
+  (set! nt/subprocess-argument-quote-char #\")
+  (set! nt/subprocess-argument-escape-char #\\)
   unspecific)
 
 (define (os/make-subprocess filename arguments environment working-directory
@@ -518,16 +492,16 @@ USA.
       (error "Can't manipulate controlling terminal of subprocess:" ctty))
   ((ucode-primitive nt-make-subprocess 8)
    filename
-   (nt/rewrite-subprocess-arguments filename (vector->list arguments))
+   (rewrite-args filename (vector->list arguments))
    (and environment
-	(nt/rewrite-subprocess-environment (vector->list environment)))
+	(rewrite-env (vector->list environment)))
    working-directory
    stdin
    stdout
    stderr
    (vector nt/hide-subprocess-windows?)))
 
-(define (nt/rewrite-subprocess-environment strings)
+(define (rewrite-env strings)
   (let ((strings
 	 (map car
 	      (sort (map (lambda (binding)
@@ -553,7 +527,7 @@ USA.
 		(loop (cdr strings) (fix:+ index* 1))))))
       result)))
 
-(define (nt/rewrite-subprocess-arguments program strings)
+(define (rewrite-args program strings)
   ;; PROGRAM will eventually be used to determine the appropriate
   ;; escape character -- strangely enough, this depends on what
   ;; runtime library PROGRAM is linked with.
@@ -561,11 +535,10 @@ USA.
   (let ((quote-char nt/subprocess-argument-quote-char)
 	(escape-char nt/subprocess-argument-escape-char))
     (if (not quote-char)
-	(nt/rewrite-subprocess-arguments/no-quoting strings)
-	(nt/rewrite-subprocess-arguments/quoting strings
-						 quote-char escape-char))))
+	(rewrite-args/no-quoting strings)
+	(rewrite-args/quoting strings quote-char escape-char))))
 
-(define (nt/rewrite-subprocess-arguments/no-quoting strings)
+(define (rewrite-args/no-quoting strings)
   (if (pair? strings)
       (let ((result
 	     (make-string
@@ -584,27 +557,27 @@ USA.
 	result)
       ""))
 
-(define (nt/rewrite-subprocess-arguments/quoting strings
-						 quote-char escape-char)
+(define (rewrite-args/quoting strings quote-char escape-char)
   (define (analyze-arg s)
     (let ((need-quotes? #f)
 	  (n (string-length s)))
       (do ((i 0 (fix:+ i 1))
-	   (j 0
-	      (fix:+ j
+	   (j 0 (if (char=? escape-char (string-ref s i)) (fix:+ j 1) 0))
+	   (k 0
+	      (fix:+ k
 		     (let ((c (string-ref s i)))
-		       (if (or (char=? quote-char c)
-			       (char=? escape-char c))
+		       (if (char=? quote-char c)
 			   (begin
 			     (set! need-quotes? #t)
-			     2)
+			     ;; Double preceding escape chars.
+			     (fix:+ j 2))
 			   (begin
 			     (if (or (char=? #\space c)
 				     (char=? #\tab c))
 				 (set! need-quotes? #t))
 			     1))))))
 	  ((fix:= i n)
-	   (cons (if need-quotes? (fix:+ j 2) j)
+	   (cons (if need-quotes? (fix:+ k 2) k)
 		 need-quotes?)))))
   (let ((analyses (map analyze-arg strings)))
     (let ((result (make-string (reduce + 0 (map car analyses)))))
@@ -612,21 +585,22 @@ USA.
 	(if (cdr analysis)
 	    (begin
 	      (string-set! result index quote-char)
-	      (let ((index (do-arg-1 index s)))
+	      (let ((index (do-arg-1 (fix:+ index 1) s)))
 		(string-set! result index quote-char)
 		(fix:+ index 1)))
 	    (do-arg-1 index s)))
       (define (do-arg-1 index s)
 	(let ((n (string-length s)))
 	  (do ((i 0 (fix:+ i 1))
+	       (j 0 (if (char=? escape-char (string-ref s i)) (fix:+ j 1) 0))
 	       (index index
 		      (let ((c (string-ref s i)))
-			(if (or (char=? quote-char c)
-				(char=? escape-char c))
-			    (begin
-			      (string-set! result index escape-char)
-			      (string-set! result (fix:+ index 1) c)
-			      (fix:+ index 2))
+			(if (char=? quote-char c)
+			    (let ((index* (fix:+ index (fix:+ j 1))))
+			      ;; Double preceding escape chars.
+			      (substring-fill! result index index* escape-char)
+			      (string-set! result index* c)
+			      (fix:+ index* 1))
 			    (begin
 			      (string-set! result index c)
 			      (fix:+ index 1))))))
@@ -697,8 +671,10 @@ USA.
 (define (os/parse-path-string string)
   (let ((end (string-length string))
 	(substring
-	 (lambda (string start end)
-	   (pathname-as-directory (substring string start end)))))
+	 (let ((cs (char-set-invert (char-set #\"))))
+	   (lambda (string start end)
+	     (pathname-as-directory (string-trim (substring string start end)
+						 cs))))))
     (let loop ((start 0))
       (if (< start end)
 	  (let ((index (substring-find-next-char string start end #\;)))
